#ifndef cGAME_HEADER
#define cGAME_HEADER

#include "cBoard.h"

typedef double (cStrat) (const cPlayer player, cBoard& board, cMatrix<double>& mGrid, cMatrix<bool>& mValid);
typedef double (cDecide)(const cPlayer player, cBoard& board, int depth, cMatrix<double>& mGrid, cMatrix<bool>& mValid, cStrat* eval, int& r_out, int& c_out);

//if the is a input, InitGrid make it symmetrical
cMatrix<double> InitGrid(cMatrix<double> input = cMatrix<double>());
cMatrix<double> InitGrid2(cMatrix<double> input = cMatrix<double>());
cMatrix<double> InitGrid3(cMatrix<double> input = cMatrix<double>());

//handling the evaluation function
int Mobility(const cPlayer player, cBoard& board, cMatrix<bool>& mValid);
double Position(const cPlayer player, cBoard& board, cMatrix<double>& mGrid);
double Position2(const cPlayer player, cBoard& board, cMatrix<double>& mGrid);
double Position3(const cPlayer player, cBoard& board, cMatrix<double>& mGrid);
double Evaluate(const cPlayer player, cBoard& board, cMatrix<double>& mGrid, cMatrix<bool>& mValid);
double Evaluate2(const cPlayer player, cBoard& board, cMatrix<double>& mGrid, cMatrix<bool>& mValid);
double Evaluate3(const cPlayer player, cBoard& board, cMatrix<double>& mGrid, cMatrix<bool>& mValid);
double Evaluate4(const cPlayer player, cBoard& board, cMatrix<double>& mGrid, cMatrix<bool>& mValid);

double Decide(const cPlayer player, cBoard& board, int depth, cMatrix<double>& mGrid, cMatrix<bool>& mValid, cStrat* eval, int& r_out, int& c_out);
double NoRandDecide(const cPlayer player, cBoard& board, int depth, cMatrix<double>& mGrid, cMatrix<bool>& mValid, cStrat* eval, int& r_out, int& c_out);


class cGame
{
public:
	cGame()
	{
		NewGame();

		// v 1.03
		mGrid1 = InitGrid2();
		mGrid2 = InitGrid3();

		fEval1 = &Evaluate2;
		fEval2 = &Evaluate3;

		// v 1.10 --- bad. comp give away corner easily
//		fEval1 = &Evaluate4;
//		fEval2 = &Evaluate4;

		fDecide1 = &Decide;
		fDecide2 = &Decide;
		

		/* an attempt to improve v1.03, fail
		mGrid1 = InitGrid3();
		mGrid2 = InitGrid2();

		fEval1 = &Evaluate3;
		fEval2 = &Evaluate2;

		fDecide1 = &Decide;
		fDecide2 = &Decide;
		*/

		nDepth = 2;
	};

	//datqa
	cBoard board;
	cPlayer player;

	//make a move
	bool MakeMove(int r, int c);
	
	//reset to new game
	void NewGame(bool reset_history = true)
	{
		player = gBLACK;
		board = cBoard();
		bEnd = false;
		history.erase(history.begin(), history.end());
		//if (reset_history) history.erase(history.begin(), history.end());
	}

	//intentionally don't use operator>>
	void OutGame();

	//I/O
	bool Load(string fn);
	bool Save(string fn);
	
	//Comp player 1 plays
	pair<int, int> Play1();
	//Comp player 2 plays
	pair<int, int> Play2();
	//Comp player 3 plays, a combination of 1 and 2
	pair<int, int> Play3();

	//game end
	bool End(){return bEnd;};

	//attempt to find a better mGrid then mGrid1 given the current one, by having p1 and p2 play against each other
	bool SearchGrid(const clock_t duration);

	//set depth
	void SetDepth(int depth){nDepth = depth;};

	//if p == gBLACK, undo last move, if p == player, undo till player's move
	bool Undo(cPlayer p);
	//reverse of undo
	bool Redo(cPlayer p);

	//return the lastmove
	pair<int, int> LastMove(void);
	
private:

	//return true if sucessfully switch, false if end of game
	bool Switch();

	//end the game, output result, set flag
	void Endgame();

	//figure out a reasonable depth
	int nDepth;
	int Depth();

	//the positional computation for player 1 and 2
	cMatrix<double> mGrid1;
	cMatrix<double> mGrid2;

	//the evaluation function for both player 1 and 2
	cStrat* fEval1;
	cStrat* fEval2;

	//the decision tree function
	cDecide* fDecide1;
	cDecide* fDecide2;

	//record down the history
	struct cAction
	{
		int r;
		int c;
		cPlayer p;
	};
	vector<cAction> history;

	//record if the game ends
	bool bEnd;
};

const double INFINITY = 64000000;

double Decide(const cPlayer player, cBoard& board, int depth, cMatrix<double>& mGrid, cMatrix<bool>& mValid, cStrat* eval, int& r_out, int& c_out);
//double Decide(const cPlayer player, cBoard& board, int depth, cMatrix<double>& mGrid, cMatrix<bool>& mValid, int& r_out, int& c_out);

struct cMove
{
	int r;
	int c;
	double priority;

	cMove(int _r, int _c, double _priority)
	{
		r = _r; c = _c; priority = _priority;
	};
};

//handling the priority of move by decide
bool CompareMove(const cMove& m1, const cMove& m2);
vector<cMove> GetOrderMove(const cPlayer& player, cBoard& board, cMatrix<bool>& mValid, cMatrix<double>& mGrid);

#endif
